! *************************************************************************************************
! *** SUE *****************************************************************************************
! *************************************************************************************************
! *** sue_main.f90 ********************************************************************************
! *************************************************************************************************



PROGRAM sue_main


  USE sue_config
  USE sue_lib
  USE sue_box
  USE sue_sed
  USE sue_sedcor
  USE sue_data
  USE sue_gaia
  IMPLICIT NONE


! *************************************************************************************************
! *** MODEL VARIABLE DECLARATION ******************************************************************
! *************************************************************************************************


! *** model VERSION details ***
  CHARACTER(LEN=4)::program_version_num  = "4.00"
  CHARACTER(LEN=8)::program_version_date = "09/10/00"
  CHARACTER(LEN=2)::colnum
  CHARACTER(LEN=2)::laynum

! *** primary variables ***
! loop variables
  INTEGER::c,l,m,o,p
! time loop variables
  INTEGER::t_init          = 1
  INTEGER::t_max           = 1000
  INTEGER::t_loop          = 0
  INTEGER::t_loop_init     = 1
  INTEGER::t_loop_max      = 1000
  REAL(KIND=fp_1)::dt      = 1.0
  REAL(KIND=fp_1)::dt_sub  = 0.0
  REAL(KIND=fp_1)::dt_dt   = 0.0
  REAL(KIND=fp_1)::dt_loop = 1.0
! ocean box descriptors
  REAL(KIND=fp_1),DIMENSION(n_col,n_dis)::sur_disconst      ! ocean surface carbonate system kinetic constants
  REAL(KIND=fp_1),DIMENSION(n_col,n_sscc)::sur_carbchem     ! sea surface carbonate chemistry
  TYPE(ocean_circulation),DIMENSION(n_wflux_max)::ocean_cir ! ocean circulation
  TYPE(ocean_atm_tracer)::oatracer                          ! combined ocean-atmosphere tracer
  REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_phys)::phys
  REAL(KIND=fp_1),DIMENSION(n_col,n_bionp)::bio_np          ! new production
  REAL(KIND=fp_1),DIMENSION(n_col,n_bionp)::bio_np_fixed    ! new production (fixed magnitude)
  REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_t_int_max)::o_conv_adj ! ocean convective adjustments
  REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_t_int_max)::o_conv_adj_fixed ! ocean convective adjustments (fixed)
  REAL(KIND=fp_1)::sealevel
  REAL(KIND=fp_1)::ocir
  REAL(KIND=fp_1),DIMENSION(n_col)::SST                     ! surface ocean temperature
  REAL(KIND=fp_1),DIMENSION(n_col)::SSS                     ! surface ocean salinity
  REAL(KIND=fp_1),DIMENSION(n_col)::SSPO4                   ! surface ocean [PO4]
  REAL(KIND=fp_1),DIMENSION(n_tracer)::fterr
  REAL(KIND=fp_1),DIMENSION(n_tracer)::fneri
  REAL(KIND=fp_1),DIMENSION(n_col)::fdust                   ! dust flux to surface ocean
  REAL(KIND=fp_1),DIMENSION(n_col)::fdust_Fe_dis            ! solubility of Fe in deposited dust
  REAL(KIND=fp_1),DIMENSION(n_col)::fdust_survar            ! surface glacial-interglaical variabilty factor
  REAL(KIND=fp_1),DIMENSION(n_col)::frefrac                 ! detrital component to the deep ocean
  REAL(KIND=fp_1)::fSi                                      ! Si (weathering) flux to the ocean
  TYPE(ocean_atm_tracer)::foat_gaia                         ! gaian ocean-atmosphere fluxes
! Present InterGlacial (PIG) state arrays
  TYPE(ocean_atm_tracer)::PIG_oatracer
  REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_phys)::PIG_phys
  REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_t_int_max)::PIG_o_conv_adj
! gaia tracer audit
  REAL(KIND=fp_1),DIMENSION(n_tracer)::tracer_tot,tracer_tot_init
  REAL(KIND=fp_1),DIMENSION(n_tracer)::dtracer_tot,dtracer_tot_per
  REAL(KIND=fp_1),DIMENSION(n_tracer)::d_dtracer_tot
! temporary or incremental variables
  TYPE(ocean_atm_tracer)::oatracer_annual                   ! ocean-atmosphere tracer annual average
  REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_phys)::temp_phys
  REAL(KIND=fp_1),DIMENSION(n_col,n_lay,n_t_int_max)::d_o_conv_adj
! sediment sub-system (i.e., integral sediment system)
  REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_sed_tot,n_sed_com_par)::sed ! 
  REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sed_top       ! top sedimentary layer
  REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_sed_par)::sed_par           ! 
  REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_sed_fnp)::sed_fnp           ! n.p. flux to sediments (yr-1)
  REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_dis)::sed_disconst          ! 
  REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_tracer)::sed_tracer         ! average sediment ocean chemistry
  REAL(KIND=fp_1),DIMENSION(n_col,n_sed_depth,n_sed_com_par)::sed_fdis      ! sediment dissolution fluxes
! misc data variables
  REAL(KIND=fp_1),DIMENSION(n_col,n_bath_depth,n_bath_par)::bath_data    ! ocean bathymetry data

! *** glacial/interglacial cycle experiment variables ***
! time loop variables
  INTEGER::t_ice       = 0
  INTEGER::t_spin      = 0
! physical and chemical forcing data
  REAL(KIND=fp_1),DIMENSION(0:n_kyr)::spec_d18Onorm         ! normalized SPECMAP d18O signal (0.0 to -1.0)
  REAL(KIND=fp_1),DIMENSION(0:n_kyr)::sig_d18O              ! (SPECMAP) d18O signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr)::sig_ocir              ! ocean circulation signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr)::sig_sealevel          ! sealevel signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_tracer)::sig_terr     ! terrestrial tracer signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_tracer)::sig_fterr    ! terrestrial tracer flux signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_tracer)::sig_neri     ! neritic sediments tracer signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_tracer)::sig_fneri    ! neritic sediments tracer flux signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col)::sig_fdust       ! terregeneous atmospheric dust deposition signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr)::sig_fSi               ! Si (weathering) flux I/P signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col)::sig_swind       ! wind speed signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col)::sig_SST         ! surface ocean temperature signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col)::sig_SSS         ! surface ocean salinity signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col)::sig_SSPO4       ! surface ocean [PO4] signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col)::sig_I           ! solar insolation signal
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col)::sig_seaice_min  ! sea ice extent signal (summer minimum)
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col)::sig_seaice_max  ! sea ice extent signal (winter maximum)
! glacial/interglacial cycle storage variables
  TYPE(ocean_atm_tracer),DIMENSION(0:n_kyr)::kyr_oatracer
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col,n_lay,n_phys_kyr)::kyr_phys
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col,n_bionp)::kyr_bionp
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col,n_lay,n_t_int_max)::kyr_o_conv_adj
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col,n_sed_depth,n_sed_fnp)::kyr_sed_fnp
  REAL(KIND=fp_1),DIMENSION(0:n_kyr,n_col,n_sed_depth,n_sed_com_par)::kyr_sed_fdis
! spin-up variables
  TYPE(ocean_atm_tracer),DIMENSION(n_spin)::spin_oatracer
  REAL(KIND=fp_1),DIMENSION(n_spin,n_col,n_lay,n_phys_spin)::spin_phys
  REAL(KIND=fp_1),DIMENSION(n_spin,n_col,n_bionp)::spin_bionp
  REAL(KIND=fp_1),DIMENSION(n_spin,n_col,n_lay,n_t_int_max)::spin_o_conv_adj
  REAL(KIND=fp_1),DIMENSION(n_spin,n_col,n_sed_depth,n_sed_fnp)::spin_sed_fnp
  REAL(KIND=fp_1),DIMENSION(n_spin,n_col,n_sed_depth,n_sed_com_par)::spin_sed_fdis
! gaia meta-system
  TYPE(ocean_atm_tracer),DIMENSION(0:n_kyr)::kyr_foat_gaia ! gaian ocean-atmosphere fluxes

! *************************************************************************************************


! *************************************************************************************************
! *** MODEL START-UP ******************************************************************************
! *************************************************************************************************

! *** display model start-up message ***
  PRINT*,""
  PRINT*,"******************************************************************************"
  PRINT*,"***                    SUE: global carbon cycle model                      ***"
  PRINT*,"******************************************************************************"
  PRINT*,""

! *** output status message: program version and ocean configuration ***
  colnum = conv_num_char(n_col)
  laynum = conv_num_char(n_lay)
  PRINT*,"SUE >> ocean box configuration         : "//colnum//" columns by "//laynum//" layers"
  PRINT*,"SUE >> program version & creation date : "//program_version_num//" ("//program_version_date//")"
  PRINT*,""
  PRINT*,"SUE >> initializing ..."
  PRINT*,""

! *** load program configuration parameter data-set ***
  PRINT*,"       * loading model configuration data"
  CALL load_par()
! adjust integration time variables
  dt_sub  = 1.0 / REAL(par_n_t_int,KIND=fp_1)
  dt_dt   = REAL(par_t_dt,KIND=fp_1) / REAL(t_loop_max,KIND=fp_1)

! *************************************************************************************************


! *************************************************************************************************
! *** SET DEFAULT DATA AND INITIALIZE SYSTEM ******************************************************
! *************************************************************************************************


! *** allocate variable size arrays ***
  PRINT*,"       * allocating array space"
! allocate size of bioturbation mixing rate array
! NOTE: the index=0 value in the array is now used for the top sediment to sediment stack mixing rate value
  ALLOCATE(sed_mix_k(0:n_sed_mix),STAT=error)
! allocate size of user-definable sediment (core) arrays
  ALLOCATE(sedcor(n_sedcor,n_sedcor_tot,n_sed_com_par),STAT=error)
  ALLOCATE(sedcor_top(n_sedcor,n_sed_com_par),STAT=error)
  ALLOCATE(sedcor_par(n_sedcor,n_sed_par),STAT=error)
  ALLOCATE(sedcor_disconst(n_sedcor,n_dis),STAT=error)
  ALLOCATE(sedcor_tracer(n_sedcor,n_tracer),STAT=error)
  ALLOCATE(sedcor_fnp(n_sedcor,n_sed_fnp),STAT=error)
  ALLOCATE(sedcor_fdis(n_sedcor,n_sed_fnp),STAT=error)
! allocate size of look-up tables
  IF (sedopt(sopt_sed_diagen_lookup_CaCO3) == yes) THEN
    ALLOCATE(lookup_sed_dis_cal( &
      & lookup_i_D_min:lookup_i_D_max, &
      & lookup_i_dCO3_min:lookup_i_dCO3_max, &
      & lookup_i_frac_min:lookup_i_frac_max, &
      & lookup_i_fCorg_min:lookup_i_fCorg_max &
      & ),STAT=error)
    ALLOCATE(lookup_sed_dis_arg( &
      & lookup_i_D_min:lookup_i_D_max, &
      & lookup_i_dCO3_min:lookup_i_dCO3_max, &
      & lookup_i_frac_min:lookup_i_frac_max, &
      & lookup_i_fCorg_min:lookup_i_fCorg_max &
      & ),STAT=error)
  ENDIF
  IF (sedopt(sopt_sed_diagen_lookup_opal) == yes) THEN
    ALLOCATE(lookup_sed_dis_opal( &
      & lookup_i_opalpc_min:lookup_i_opalpc_max, &
      & lookup_i_concSi_min:lookup_i_concSi_max, &
      & lookup_i_T_min:lookup_i_T_max, &
      & lookup_i_KSi0_min:lookup_i_KSi0_max, &
      & lookup_i_opaltorefrac_min:lookup_i_opaltorefrac_max &
      & ),STAT=error)
  ENDIF

! *** check for problems allocating array space ***
  IF (error /= 0) THEN
    PRINT*,"FATAL ERROR [sue_main.f90, main]:"
    PRINT*,"Array space could not be allocated"
    STOP
  ENDIF

! *** initialize arrays ***
  PRINT*,"       * initializing arrays"
! primary variables
  oatracer%ioat_t = 0.0
  oatracer%ioat_a = 0.0
  phys            = 0.0
! PIG variables
  PIG_oatracer%ioat_t = 0.0
  PIG_oatracer%ioat_a = 0.0
  PIG_phys            = 0.0
! forcing signals
  sig_d18O       = 0.0
  sig_sealevel   = 0.0
  sig_terr       = 0.0
  sig_fterr      = 0.0 
  sig_neri       = 0.0
  sig_fneri      = 0.0 
  sig_fdust      = 0.0
  sig_fSi        = 0.0
  sig_swind      = 0.0
  sig_SST        = 0.0
  sig_SSS        = 0.0
  sig_I          = 0.0
  sig_seaice_min = 0.0
  sig_seaice_max = 0.0
! tracer audit
  tracer_tot      = 0.0
  tracer_tot_init = 0.0
  dtracer_tot     = 0.0
  dtracer_tot_per = 0.0
  d_dtracer_tot   = 0.0
! user-defined sediment core arrays
  sedcor          = 0.0
  sedcor_top      = 0.0
  sedcor_par      = 0.0
  sedcor_disconst = 0.0
  sedcor_tracer   = 0.0
  sedcor_fnp      = 0.0
! new production
  bio_np       = 0.0
  bio_np_fixed = 0.0
! insolation and seaice seasonal cycles
  seas_t_np_insol   = 0.0
  seas_t_np_seaice  = 0.0
  seas_t_int_insol  = 0.0
  seas_t_int_seaice = 0.0
! kyr arrays
  kyr_phys       = 0.0
  kyr_bionp      = 0.0
  kyr_o_conv_adj = 0.0
  kyr_sed_fnp    = 0.0
  kyr_sed_fdis   = 0.0
! spin arrays
  spin_phys       = 0.0
  spin_bionp      = 0.0
  spin_o_conv_adj = 0.0
  spin_sed_fnp    = 0.0
  spin_sed_fdis   = 0.0
! misc
  o_conv_adj             = 0.0
  oatracer_annual%ioat_t = 0.0
  oatracer_annual%ioat_a = 0.0

! *** load in model configuration description data ***
  PRINT*,"       * loading model description data"
! ocean boxes
  CALL load_ocean_par(ocean_cir,PIG_oatracer,PIG_phys,o_conv_adj_fixed)
! check ocean mass balance
  CALL chk_boxmassbal(ocean_cir)
! atmospheric box
  CALL load_atm_par(PIG_oatracer)
! load sediment core array data
  IF (runopt(ropt_sedcor) == yes) CALL load_sedcor_par(sedcor_par)
! load in bathymetry data-set
  CALL load_bathdata(bath_data)
! load in sediment depth data-set
  CALL load_sed_depth(sed_par)
! load in sediment sub-layer mixing rate profile
  CALL load_sed_mix_k(sed_mix_k)
! load in fixed magnitude new production fluxes (if requrested)
  IF ((bioopt(bopt_np_alt) == "2") .OR. (bioopt(bopt_np_alt) == "3")) THEN
    CALL load_bionp_fixed(bio_np_fixed)
  ELSE
    bio_np_fixed = 0.0
  ENDIF
! load in dust Fe solubility distributions
  CALL load_dust_Fe_dis(fdust_Fe_dis)
! load in surface ocean aeolian variability field
  CALL load_dust_survar(fdust_survar)

! *** load in model look-up table data ***
! load in CaCO3 sediment diagenesis look-up tables
  IF ((runopt(ropt_sed) == yes) .AND. (sedopt(sopt_sed_diagen_lookup_CaCO3) == yes)) THEN
! *** output status message: confirm that program is operating ***
    PRINT*,"       * loading sediment diagenesis look-up table data : CaCO3"
    CALL load_sed_dis_lookup_CaCO3()
  ENDIF
! load in opal sediment diagenesis look-up tables
  IF ((runopt(ropt_sed) == yes) .AND. (sedopt(sopt_sed_diagen_lookup_opal) == yes)) THEN
! *** output status message: confirm that program is operating ***
    PRINT*,"       * loading sediment diagenesis look-up table data : opal"
    CALL load_sed_dis_lookup_opal()
  ENDIF

! *** load in model forcing signal data-sets ***
  PRINT*,"       * loading forcing signal data"
! ocean circulation change
  CALL load_sig_ocir(sig_ocir)
! wind speed change
  CALL load_sig_swind(sig_swind)
! sealevel change
  CALL load_sig_sealevel(sig_sealevel)
! ocean surface temperature change
  CALL load_sig_SST(sig_SST)
! ocean surface salinity change
  CALL load_sig_SSS(sig_SSS)
! ocean surface [PO4] change
  CALL load_sig_SSPO4(sig_SSPO4)
! solar insolation change
  CALL load_sig_I(sig_I)
! sea ice extent change
  CALL load_sig_seaice(sig_seaice_min,sig_seaice_max)
! terrestrial C storage signal
  CALL load_sig_terr_C(sig_terr)
! neritic sediments CaCO3 storage signal
  CALL load_sig_neri_CaCO3(sig_neri)
! neritic sediments opal storage signal
  CALL load_sig_neri_opal(sig_neri)
! terregeneous dust signal
  CALL load_sig_fdust(sig_fdust)
! Si (weathering) flux I/P signal
  CALL load_sig_fSi(sig_fSi)
! oceanic d18O signal
  CALL load_specmap_d18O(sig_d18O)

! *** initialize default model state ***
  PRINT*,"       * initializing default model state"
! calculate default (PIG) ocean surface areas (if required)
  IF (miscopt(mopt_Alatlon) == yes) CALL calc_Alatlon(PIG_phys)
! set default (PIG) ocean surface temperature and salinity
  PIG_oatracer%ioat_t(:,1,it_T) = sig_SST(0,:)
  PIG_oatracer%ioat_t(:,1,it_S) = sig_SSS(0,:)
! set default (PIG) ocean [H+]
  PIG_phys(:,:,ip_H) = 7.0E-09
! finish determining PIG physics variables:
! NOTE: the 'PIG_phys' array is used twice,
!       once as input only into the subroutine to set basic ocean dimensions,
!       and a second time as in/out to be updated with full ocean dimension data
  PIG_phys(:,:,ip_T) = PIG_oatracer%ioat_t(:,:,it_T)
  CALL calc_bathdata_box(bath_data,PIG_phys,sig_sealevel(PIG_t_ice),PIG_phys)
! copy variables from master PIG array into temporary array
  phys = PIG_phys
! check the fractional mass exchange
  CALL chk_boxfracmassex(dt_sub,ocean_cir,PIG_phys)
! initialize ocean-atmosphere tracer values 'oatracer'
  oatracer%ioat_t(:,:,:) = PIG_oatracer%ioat_t(:,:,:)
  oatracer%ioat_a(:)     = PIG_oatracer%ioat_a(:)
! initialize annual mean tracer array
  oatracer_annual = oatracer

! *** calculate neritic tracer storage ***
! neritic sediments CaCO3 storage change
  IF (gaiaopt(gopt_ext_sig_CaCO3) == no) THEN
    PRINT*,"       * calculating neritic CaCO3 storage signals"
    CALL calc_sig_neri_CaCO3(sig_neri,bath_data,sig_sealevel,PIG_phys)
  ENDIF
! neritic sediment opal storage change
  IF (gaiaopt(gopt_ext_sig_opal) == no) THEN
    PRINT*,"       * calculating neritic opal storage signals"
    CALL calc_sig_neri_opal(sig_neri,bath_data,sig_sealevel,PIG_phys)
  ENDIF

! *** calculate geochemical forcings ***
  PRINT*,"       * calculating geochemical flux forcing signals"
! calculate terrestrial tracer flux signal
  sig_fterr = calc_sig_fterr(sig_terr)
! calculate neritic sediment flux signal
  sig_fneri = calc_sig_fneri(sig_neri)

! *** initialize sediment sub-system ***
  PRINT*,"       * initializing sediment system"
! set integral sediment sub-system
  CALL init_sed_layers_default(sed,sed_top)
  CALL init_sed_par(sed_par)
  CALL calc_bathdata_sed(bath_data,sed_par,PIG_phys)
! set user-definable core sediment sub-system
  IF (runopt(ropt_sedcor) == yes) THEN
    CALL init_sedcor_layers_default(sedcor,sedcor_top)
    CALL init_sedcor_par(sedcor_par,phys)
  ENDIF

! *** initialize tracer audit ***
  PRINT*,"       * initializing tracer audit"
! compile initial geochemical inventory of ocean-atmosphere system
  tracer_tot_init = calc_oatracer_tot(oatracer,PIG_phys)

! *** biological new production initialization ***
  PRINT*,"       * initializing biological new production"
! calculate Redfield Ratios
  CALL calc_rednp()
! set fixed magnitude new production fluxes (if requrested)
  IF ((bioopt(bopt_np_alt) == "2") .OR. (bioopt(bopt_np_alt) == "3")) THEN
    CALL set_bionp_fixed(bio_np_fixed)
  END IF

! *** automatic load/save (chain) model state initialization ***
  IF (runopt(ropt_chain_modelstate) == yes) THEN
    runopt(ropt_load_modelstate) = yes
    runopt(ropt_save_modelstate) = yes
  ENDIF

! *************************************************************************************************



! *************************************************************************************************
! *** SIMULATION RUN ******************************************************************************
! *************************************************************************************************


! *** SPIN-UP *************************************************************************************

  IF (runopt(ropt_load_modelstate) == yes) THEN
    t_spin_max = 0
  ENDIF

! *** output status message ***
  PRINT*,""
  PRINT*,"SUE >> spin-up ..."
  PRINT*,""
  PRINT*,"       *    time     pCO2    d13C  calsat  argsat     [CO2]     [alk]     [Si]"

! *** set system forcings (at kyr resolution) ***
! set initial value of 't_ice'
! NOTE: t_ice time is measured in kyr BEFORE PRESENT
  t_ice = t_ice_init
! prescribe d18O
  CALL set_d18O(sig_d18O(t_ice),oatracer)
  CALL set_d18O(sig_d18O(t_ice),oatracer_annual)
! initialize local loop variables
  o_conv_adj = 0.0
! set the initial system forcing values for current kyr time-step
  ocir                      = sig_ocir(t_ice)
  sealevel                  = sig_sealevel(t_ice)
  SST(:)                    = sig_SST(t_ice,:)
  SSS(:)                    = sig_SSS(t_ice,:)
  SSPO4(:)                  = sig_SSPO4(t_ice,:)
  phys(:,:,ip_T)            = oatracer%ioat_t(:,:,it_T)
  phys(:,1,ip_I)            = sig_I(t_ice,:)
  phys(:,1,ip_seaice_min)   = sig_seaice_min(t_ice,:)
  phys(:,1,ip_seaice_max)   = sig_seaice_max(t_ice,:)
  phys(:,1,ip_swind)        = sig_swind(t_ice,:)
  fterr                     = sig_fterr(t_ice,:)
  fneri                     = sig_fneri(t_ice,:)
  fdust                     = sig_fdust(t_ice,:) * fdust_survar(:) + &
    &                         sig_fdust(t_ice_init,:) * (1.0 - fdust_survar(:))
  frefrac                   = sig_fdust(t_ice,:)
  fSi                       = sig_fSi(t_ice)
! calculate insolation cycle
  CALL calc_seas_insol(phys)
! calculate seaice cycle
  CALL calc_seas_seaice(phys)
! calculate from sealevel signal; time-dependent ocean box depths, surface areas, volumes and masses
  temp_phys = PIG_phys
  CALL calc_bathdata_box(bath_data,phys,sealevel,PIG_phys)
! adjust value of 'tracer' to account for different ocean volume at t_ice_init
  CALL scale_tracer(phys(:,:,ip_V),temp_phys(:,:,ip_V),oatracer%ioat_t)
  CALL scale_tracer(phys(:,:,ip_V),temp_phys(:,:,ip_V),oatracer_annual%ioat_t)
! adjust tracer inventory of ocean and atmosphere to account for inventory differences from PIG values 
  foat_gaia = calc_foat_gaia_spin_init(sig_fterr,sig_fneri,phys)
  CALL couple_gaia(dt_loop,oatracer,phys,foat_gaia,d_dtracer_tot)
  dtracer_tot = dtracer_tot + d_dtracer_tot
! calculate spin-up gaian fluxes
  foat_gaia = calc_foat_gaia_spin(fterr,fneri,fdust,fdust_Fe_dis,fSi,phys)
! *** spin-up over t_spin_max kyr ***
! NOTE: adjust atmosphere to account for excess carbon stored in terrestrial ecosystems,
! and the ocean to account for excess CaCO3 stored in coral reefs,
! during the 1st spin-up kyr only,
! then set C and CaCO3 transfer for remaining (repeated) spin-up kyrs to zero
  DO t_spin = t_spin_init,t_spin_max
! store data
! NOTE: the current annual mean 'tracer' value is stored ('tracer_annual'), not 'tracer' itself
    spin_oatracer(t_spin)         = oatracer_annual
    spin_phys(t_spin,:,:,:)       = phys(:,:,1:n_phys_spin)
    spin_bionp(t_spin,:,:)        = bio_np
    spin_o_conv_adj(t_spin,:,:,:) = o_conv_adj
    spin_sed_fnp(t_spin,:,:,:)    = sed_fnp
    spin_sed_fdis(t_spin,:,:,:)   = sed_fdis
! initialize local loop variables
    o_conv_adj = 0.0
! audit the geochemical inventory of the entire system
    CALL audit_gaia(dtracer_tot,tracer_tot_init,oatracer,phys,dtracer_tot_per)

! *** sub 1kr equilibrium loop ***
! integrate over 1000 yrs with 1 yr time steps
    DO t_loop = t_loop_init,t_loop_max
! display some results
      IF (MOD((t_loop - 1),t_print) == 0) THEN
	CALL print_spinstepdata(oatracer_annual,phys,bio_np,t_spin,t_loop - 1)
      ENDIF
      IF (MOD((t_loop - 1),par_t_dt) == 0) THEN
! re-calculate surface ocean carbonate equilibrium constants
! NOTE: this is necessary because it will take the salinity distribution in the ocean
! some time to equilibriate after the initial ocean volume changes,
! which will have a knock-on effect on surface ocean carbonate equilibrium constants
	CALL calc_sur_disconst(sur_disconst,phys(:,1,:),oatracer_annual%ioat_t(:,1,it_S))
! re-calculate surface ocean carbonate equilibrium
	CALL calc_sur_carbchem_Hinout(sur_carbchem,phys(:,1,:),oatracer_annual%ioat_t(:,1,:),sur_disconst)
! re-calculate calcite and aragonite saturation depths
	CALL calc_D_sat(phys,oatracer_annual%ioat_t)
! to update integral sediment sub-system calculate:
! (1) sediment depths
! (2) sediment / ocean layer connectivity
! (3) average sediment tracer values
! (4) sediment carbonate equilibrium constants
! (5) carbonate equilibrium
	CALL update_sed_bath(sed_par,sealevel)
	CALL calc_sed_lay(sed_par,phys)
	CALL calc_sed_tracer(sed_par,sed_tracer,oatracer_annual%ioat_t,phys)
	CALL calc_sed_disconst(sed_disconst,sed_par,sed_tracer)
	CALL calc_sed_carbchem(sed_par,sed_disconst,sed_tracer)
! to update user-defined sediment cores calculate:
! (1) sediment depths
! (2) sediment / ocean layer connectivity
! (3) sediment tracer values
! (4) sediment carbonate equilibrium constants
! (5) carbonate equilibrium
	IF (runopt(ropt_sedcor) == yes) THEN
	  CALL update_sedcor_bath(sedcor_par,phys,sealevel)
	  CALL calc_sedcor_lay(sedcor_par,phys)
	  CALL calc_sedcor_tracer(sedcor_par,sedcor_tracer,oatracer_annual%ioat_t,phys)
	  CALL calc_sedcor_disconst(sedcor_disconst,sedcor_par,sedcor_tracer)
	  CALL calc_sedcor_carbchem(sedcor_par,sedcor_disconst,sedcor_tracer)
	ENDIF
! to update new production remineralization calculate:
! (1) new production fractions for each sediment layer
! (2) remineralization function values calculation
	CALL calc_Afrac_np(sed_par,phys)
	CALL calc_remin_frac_sed(oatracer_annual%ioat_t,sed_tracer,phys,sed_par)
	IF (runopt(ropt_sedcor) == yes) THEN
	  CALL calc_remin_frac_sedcor(oatracer%ioat_t,sedcor_tracer,phys,sedcor_par)
	END IF
      ENDIF
! solve intra-ocean and ocean-atmosphere interactions
! force d18O
      CALL set_ocir(ocir,ocean_cir(1:par_n_wflux))
      CALL update_ocnatm(dt_sub,oatracer,foat_gaia, &
	& phys,o_conv_adj_fixed,ocean_cir(1:par_n_wflux),sur_disconst,sed_par,sedcor_par, &
	& bio_np_fixed,fdust, &
	& SST,SSS,SSPO4, &
	& oatracer_annual,sed_fnp,sedcor_fnp,bio_np, &
	& d_o_conv_adj,d_dtracer_tot)
      CALL set_d18O(sig_d18O(t_ice),oatracer)
      CALL set_d18O(sig_d18O(t_ice),oatracer_annual)
! update tracer audit changes
      dtracer_tot = dtracer_tot + d_dtracer_tot
! update mean ocean convective adjustments
      o_conv_adj = o_conv_adj + d_o_conv_adj / 1000.0
! asynchronously couple ocean-atmosphere and sediment sub-system (if requested)
! update radiocarbon sediment clock (if requested)
! NOTE: force d18O
      IF (runopt(ropt_sed) == yes) THEN
	CALL couple_sed(dt,(t_ice_init + t_spin_max - t_spin),t_loop, &
	  & oatracer%ioat_t,sed_tracer,sed_fnp,frefrac,phys, &
	  & sed,sed_top,sed_par,sed_mix_k,dtracer_tot,sed_fdis,sur_carbchem)
	CALL set_d18O(sig_d18O(t_ice),oatracer)
	IF (sedopt(sopt_sed_14C) == yes) THEN
	  CALL update_sed_14C(sed,sed_top)
	ENDIF
      ENDIF
! asynchronously couple ocean-atmosphere and user-defined sediment cores (if requested)
! update radiocarbon sediment clock (if requested)
      IF (runopt(ropt_sedcor) == yes) THEN
	CALL couple_sedcor(dt_y,(t_ice_init + t_spin_max - t_spin),t_loop, &
	  & oatracer%ioat_t,sedcor_tracer,sedcor_fnp,frefrac,phys, &
	  & sedcor,sedcor_top,sedcor_par,sed_mix_k,dtracer_tot,sedcor_fdis,sur_carbchem)
	IF (sedopt(sopt_sedcor_14C) == yes) THEN
	  CALL update_sedcor_14C(sedcor,sedcor_top)
	ENDIF
      ENDIF
! check for the occurance of ocean anoxia
! NOTE: if any of the ocean boxes are anoxic:
!       (a) deduct the oxygen deficit from the atmosphere
!       (b) reset [O2] to zero for that particular box
!       (c) reset d18O
      IF (MINVAL(oatracer%ioat_t(:,:,it_O2)) < par_remin_O2_min) THEN
	DO c = 1,n_col
	  DO l = 1,n_lay
	    IF (oatracer%ioat_t(c,l,it_O2) < par_remin_O2_min) THEN
	      IF (runopt(ropt_warn) == yes) THEN
		PRINT*,"WARNING: anoxia in ocean box"
		PRINT*,c,l,oatracer%ioat_t(c,l,it_O2) * 1.0E+06
	      ENDIF
	      program_error              = yes
	      warning(iwarn_anoxia)      = yes
	      oatracer%ioat_a(ia_pO2)    = oatracer%ioat_a(ia_pO2) + &
		& (oatracer%ioat_t(c,l,it_O2) - par_remin_O2_min) * phys(c,l,ip_M) * conv_mol_atm
	      oatracer%ioat_t(c,l,it_O2) = par_remin_O2_min
	    ENDIF
	  END DO
	END DO
	CALL set_d18O(sig_d18O(t_ice),oatracer)
      ENDIF
! update ocean physics temperature array values
      phys(:,:,ip_T) = oatracer%ioat_t(:,:,it_T)

    END DO

  END DO

! print out final spin-up results
  IF (t_spin_max > 0) THEN
    CALL print_spinstepdata(oatracer_annual,phys(:,:,:),bio_np(:,:),t_spin - 1,t_loop - 1)
  ELSE
    CALL print_spinstepdata(oatracer_annual,phys(:,:,:),bio_np(:,:),t_spin,t_loop)
  ENDIF

! *** 1KR TIME STEP MAIN PROGRAM LOOP *************************************************************

! load model state if required
  IF (runopt(ropt_load_modelstate) == yes) THEN
    PRINT*,""
    PRINT*,"SUE >> load model state"
    CALL load_all(&
      & t_ice_load, &
      & sur_disconst, &
      & sur_carbchem, &
      & ocean_cir, &
      & oatracer, &
      & phys, &
      & bio_np, &
      & bio_np_fixed, &
      & o_conv_adj, &
      & o_conv_adj_fixed, &
      & sealevel, &
      & ocir, &
      & SST, &
      & SSS, &
      & SSPO4, &
      & fterr, &
      & fneri, &
      & fdust, &
      & frefrac, &
      & fdust_Fe_dis, &
      & fSi, &
      & foat_gaia, &
      & PIG_oatracer, &
      & PIG_phys, &
      & tracer_tot, &
      & tracer_tot_init, &
      & dtracer_tot, &
      & dtracer_tot_per, &
      & d_dtracer_tot, &
      & oatracer_annual, &
      & sed, &
      & sed_top, &
      & sed_par, &
      & sed_fnp, &
      & sed_disconst, &
      & sed_tracer, &
      & sed_fdis, &
      & bath_data, &
      & spec_d18Onorm, &
      & kyr_oatracer, &
      & kyr_phys, &
      & kyr_bionp, &
      & kyr_o_conv_adj, &
      & kyr_sed_fnp, &
      & kyr_sed_fdis, &
      & spin_oatracer, &
      & spin_phys, &
      & spin_bionp, &
      & spin_o_conv_adj, &
      & spin_sed_fnp, &
      & spin_sed_fdis, &
      & kyr_foat_gaia, &
      & sedcor, &
      & sedcor_top, &
      & sedcor_par, &
      & sedcor_disconst, &
      & sedcor_tracer, &
      & sedcor_fnp, &
      & sedcor_fdis, &
      & warning &
      & )
    t_ice_init = t_ice_load
    IF (runopt(ropt_chain_modelstate) == yes) THEN
      t_ice_save = t_ice_init - t_ice_chain
    ENDIF
  ENDIF

! *** output status message ***
  PRINT*,""
  PRINT*,"SUE >> running ..."
  PRINT*,""
  PRINT*,"       *    time     pCO2    d13C  calsat  argsat     [CO2]     [alk]     [Si]"

! *** start of main program loop: set time variables ***
! note that at the end of the DO loop, the value of t_ice will be t_ice_max PLUS t_ice_inc
  DO t_ice = t_ice_init,t_ice_max,dt_ice

! *** data storage and output ***
! store kyr-step data
! NOTE: the current annual mean tracer value is stored ('tracer_annual'), not 'tracer' itself
    kyr_oatracer(t_ice)         = oatracer_annual
    kyr_bionp(t_ice,:,:)        = bio_np
    kyr_phys(t_ice,:,:,:)       = phys(:,:,1:n_phys_kyr)
    kyr_foat_gaia(t_ice)        = foat_gaia
    kyr_o_conv_adj(t_ice,:,:,:) = o_conv_adj
    kyr_sed_fnp(t_ice,:,:,:)    = sed_fnp
    kyr_sed_fdis(t_ice,:,:,:)   = sed_fdis
! save model state if required
    IF ((runopt(ropt_save_modelstate) == yes) .AND. (t_ice == t_ice_save)) THEN
      PRINT*,""
      PRINT*,"SUE >> save model state"
      CALL save_all(&
	& t_ice_save, &
	& sur_disconst, &
	& sur_carbchem, &
	& ocean_cir, &
	& oatracer, &
	& phys, &
	& bio_np, &
	& bio_np_fixed, &
	& o_conv_adj, &
	& o_conv_adj_fixed, &
	& sealevel, &
	& ocir, &
	& SST, &
	& SSS, &
	& SSPO4, &
	& fterr, &
	& fneri, &
	& fdust, &
	& frefrac, &
	& fdust_Fe_dis, &
	& fSi, &
	& foat_gaia, &
	& PIG_oatracer, &
	& PIG_phys, &
	& tracer_tot, &
	& tracer_tot_init, &
	& dtracer_tot, &
	& dtracer_tot_per, &
	& d_dtracer_tot, &
	& oatracer_annual, &
	& sed, &
	& sed_top, &
	& sed_par, &
	& sed_fnp, &
	& sed_disconst, &
	& sed_tracer, &
	& sed_fdis, &
	& bath_data, &
	& spec_d18Onorm, &
	& kyr_oatracer, &
	& kyr_phys, &
	& kyr_bionp, &
	& kyr_o_conv_adj, &
	& kyr_sed_fnp, &
	& kyr_sed_fdis, &
	& spin_oatracer, &
	& spin_phys, &
	& spin_bionp, &
	& spin_o_conv_adj, &
	& spin_sed_fnp, &
	& spin_sed_fdis, &
	& kyr_foat_gaia, &
	& sedcor, &
	& sedcor_top, &
	& sedcor_par, &
	& sedcor_disconst, &
	& sedcor_tracer, &
	& sedcor_fnp, &
	& sedcor_fdis, &
	& warning &
	& )
      IF (runopt(ropt_chain_modelstate) == yes) THEN
	t_ice_save = t_ice - t_ice_chain
      ENDIF
    ENDIF
! save timeslices if required
! (a) whole-ocean physical and chemical fields
! (b) biological new production
! (c) np time step cycle - all relevant parameters
! (d) integration time step cycle - surface ocean
! (e) integration time step cycle - biological new production
! (f) integration time step cycle - misc
! (g) sediment system rain flux (if requested)
! (h) sediment system dissolution flux  (if requested)
! (i) sediment core rain flux (if requested)
! (j) sediment core dissolution flux  (if requested)
! (k) sediment core system core-top data (if requested)
    IF ((t_ice == par_t_ice_save_1) .OR. (t_ice == par_t_ice_save_2)) THEN
      CALL save_boxxxxx(t_ice,phys,oatracer_annual,o_conv_adj)
      CALL save_bionpxx(t_ice,phys,bio_np)
      CALL save_seas_t_np_all(t_ice)
      CALL save_seas_t_int_surxx(t_ice)
      CALL save_seas_t_int_bionpxx(t_ice,phys)
      CALL save_seas_t_int_misc(t_ice)
      IF ((runopt(ropt_sed) == yes) .AND. (sedopt(sopt_save_sedfnpdata) == yes)) THEN
	CALL save_sed_fnp(t_ice,sed_fnp,sed_par)
      ENDIF
      IF ((runopt(ropt_sed) == yes) .AND. (sedopt(sopt_save_seddisdata) == yes)) THEN
	CALL save_sed_fdis(t_ice,sed_fdis,sed_par)
      ENDIF
      IF ((runopt(ropt_sedcor) == yes) .AND. (sedopt(sopt_save_sedcorfnpdata) == yes)) THEN
	CALL save_sedcor_fnp(t_ice,sedcor_fnp,sedcor_par,phys)
      ENDIF
      IF ((runopt(ropt_sedcor) == yes) .AND. (sedopt(sopt_save_sedcordisdata) == yes)) THEN
	CALL save_sedcor_fdis(t_ice,sedcor_fdis,sedcor_par,phys)
      ENDIF
      IF (runopt(ropt_sed) == yes) THEN
	CALL save_sed_coretop_data(t_ice,sed_top)
      END IF
    END IF
! audit the geochemical inventory of the entire system
    CALL audit_gaia(dtracer_tot,tracer_tot_init,oatracer,phys,dtracer_tot_per)
! test whether the end of the simulation has been reached => exit the simulation loop
    IF (t_ice == t_ice_max) THEN
      CALL print_kyrstepdata(oatracer_annual,phys,bio_np,t_ice + 1,t_loop - 1)
      EXIT
    ENDIF

! *** set system forcings (at kyr resolution) ***
! set the initial system forcing values for current kyr time-step
    ocir                      = sig_ocir(t_ice)
    sealevel                  = sig_sealevel(t_ice)
    SST(:)                    = sig_SST(t_ice,:)
    SSS(:)                    = sig_SSS(t_ice,:)
    SSPO4(:)                  = sig_SSPO4(t_ice,:)
    phys(:,1,ip_I)            = sig_I(t_ice,:)
    phys(:,1,ip_seaice_min)   = sig_seaice_min(t_ice,:)
    phys(:,1,ip_seaice_max)   = sig_seaice_max(t_ice,:)
    phys(:,1,ip_swind)        = sig_swind(t_ice,:)
    fterr                     = sig_fterr(t_ice,:)
    fneri                     = sig_fneri(t_ice,:)
    fdust                     = sig_fdust(t_ice,:) * fdust_survar(:) + &
      &                         sig_fdust(t_ice_init,:) * (1.0 - fdust_survar(:))
    frefrac                   = sig_fdust(t_ice,:)
    fSi                       = sig_fSi(t_ice)
! prescribe d18O
    CALL set_d18O(sig_d18O(t_ice),oatracer)
    CALL set_d18O(sig_d18O(t_ice),oatracer_annual)
! initialize local loop variables
    o_conv_adj = 0.0

! *** sub 1kr equilibrium loop ***
    DO t_loop = t_loop_init,t_loop_max
! display some results
      IF (MOD((t_loop - 1),t_print) == 0) THEN
	CALL print_kyrstepdata(oatracer_annual,phys,bio_np,t_ice,t_loop - 1)
      ENDIF
! adjust derived system forcings (at sub-kyr resolution):
! sealevel, ocean volumes and areas, and dependent tracer and sediment sub-systems
! NOTE: if only one step-per-kyr is selected, sealevel is incremented to the next kyr value immediately
      IF (MOD((t_loop - 1),par_t_dt) == 0) THEN
! increment system forcings, using a linear interpolation
! NOTE: do not interpolate neritic and terrestrial storage change transfer fluxes,
!       otherwise the original storage signal cannot not be faithfully reconstructed from the fluxes
	ocir     = ocir + &
	  & dt_dt * (sig_ocir(t_ice + dt_ice) - sig_ocir(t_ice))
	sealevel = sealevel + &
	  & dt_dt * (sig_sealevel(t_ice + dt_ice) - sig_sealevel(t_ice))
	SST(:)   = SST(:) + &
	  & dt_dt * (sig_SST(t_ice + dt_ice,:) - sig_SST(t_ice,:))
	SSS(:)   = SSS(:) + &
	  & dt_dt * (sig_SSS(t_ice + dt_ice,:) - sig_SSS(t_ice,:))
	SSPO4(:) = SSPO4(:) + &
	  & dt_dt * (sig_SSPO4(t_ice + dt_ice,:) - sig_SSPO4(t_ice,:))
	phys(:,1,ip_I)          = phys(:,1,ip_I) + &
	  & dt_dt * (sig_I(t_ice + dt_ice,:) - sig_I(t_ice,:))
	phys(:,1,ip_seaice_min) = phys(:,1,ip_seaice_min) + &
	  & dt_dt * (sig_seaice_min(t_ice + dt_ice,:) - sig_seaice_min(t_ice,:))
	phys(:,1,ip_seaice_max) = phys(:,1,ip_seaice_max) + &
	  & dt_dt * (sig_seaice_max(t_ice + dt_ice,:) - sig_seaice_max(t_ice,:))
	phys(:,1,ip_swind)      = phys(:,1,ip_swind) + &
	  & dt_dt * (sig_swind(t_ice + dt_ice,:) - sig_swind(t_ice,:))
	fterr(:) = fterr(:)
	fneri(:) = fneri(:)
	fdust(:) = fdust(:) + dt_dt * &
	  & ( &
	  &   ( &
	  &   sig_fdust(t_ice + dt_ice,:) * fdust_survar(:) + &
	  &   sig_fdust(t_ice_init,:) * (1.0 - fdust_survar(:)) &
	  &   ) &
	  &   - &
	  &   ( &
	  &   sig_fdust(t_ice,:) * fdust_survar(:) + &
	  &   sig_fdust(t_ice_init,:) * (1.0 - fdust_survar(:)) &
	  &   ) &
	  & )
	frefrac  = frefrac(:) + &
	  & dt_dt * (sig_fdust(t_ice + dt_ice,:) - sig_fdust(t_ice,:))
	fSi      = fSi + &
	  & dt_dt * (sig_fSi(t_ice + dt_ice) - sig_fSi(t_ice))
! calculate insolation cycle
	CALL calc_seas_insol(phys)
! calculate seaice cycle
	CALL calc_seas_seaice(phys)
! re-calculate ocean physics values
	temp_phys = phys
	CALL calc_bathdata_box(bath_data,phys,sealevel,PIG_phys)
! adjust tracer concentrations to account for changes in ocean volume
	CALL scale_tracer(phys(:,:,ip_V),temp_phys(:,:,ip_V),oatracer%ioat_t)
	CALL scale_tracer(phys(:,:,ip_V),temp_phys(:,:,ip_V),oatracer_annual%ioat_t)
! re-calculate surface ocean carbonate equilibrium constants
	CALL calc_sur_disconst(sur_disconst,phys(:,1,:),oatracer_annual%ioat_t(:,1,it_S))
! re-calculate surface ocean carbonate equilibrium
	CALL calc_sur_carbchem_Hinout(sur_carbchem,phys(:,1,:),oatracer_annual%ioat_t(:,1,:),sur_disconst)
! re-calculate calcite and aragonite saturation depths
	CALL calc_D_sat(phys,oatracer_annual%ioat_t)
! to update integral sediment sub-system calculate:
! (1a) sediment depths
! (2a) sediment / ocean layer connectivity
! (3a) average sediment tracer values
! (4a) sediment carbonate equilibrium constants
! (5a) carbonate equilibrium
	CALL update_sed_bath(sed_par,sealevel)
	CALL calc_sed_lay(sed_par,phys)
	CALL calc_sed_tracer(sed_par,sed_tracer,oatracer_annual%ioat_t,phys)
	CALL calc_sed_disconst(sed_disconst,sed_par,sed_tracer)
	CALL calc_sed_carbchem(sed_par,sed_disconst,sed_tracer)
! to update user-defined sediment cores calculate:
! (1b) sediment depths
! (2b) sediment / ocean layer connectivity
! (3b) sediment tracer values
! (4b) sediment carbonate equilibrium constants
! (5b) carbonate equilibrium
	IF (runopt(ropt_sedcor) == yes) THEN
	  CALL update_sedcor_bath(sedcor_par,phys,sealevel)
	  CALL calc_sedcor_lay(sedcor_par,phys)
	  CALL calc_sedcor_tracer(sedcor_par,sedcor_tracer,oatracer_annual%ioat_t,phys)
	  CALL calc_sedcor_disconst(sedcor_disconst,sedcor_par,sedcor_tracer)
	  CALL calc_sedcor_carbchem(sedcor_par,sedcor_disconst,sedcor_tracer)
	ENDIF
! to update new production remineralization calculate:
! (6) new production fractions for each sediment layer
! (7) remineralization function values calculation
	CALL calc_Afrac_np(sed_par,phys)
	CALL calc_remin_frac_sed(oatracer_annual%ioat_t,sed_tracer,phys,sed_par)
	IF (runopt(ropt_sedcor) == yes) THEN
	  CALL calc_remin_frac_sedcor(oatracer%ioat_t,sedcor_tracer,phys,sedcor_par)
	END IF
! update gaian fluxes
	foat_gaia = calc_foat_gaia(fterr,fneri,fdust,fdust_Fe_dis,fSi,phys)
      ENDIF
! solve intra-ocean and ocean-atmosphere interactions
! NOTE: force d18O
      CALL set_ocir(ocir,ocean_cir(1:par_n_wflux))
      CALL update_ocnatm(dt_sub,oatracer,foat_gaia, &
	& phys,o_conv_adj_fixed,ocean_cir(1:par_n_wflux),sur_disconst,sed_par,sedcor_par, &
	& bio_np_fixed,fdust, &
	& SST,SSS,SSPO4, &
	& oatracer_annual,sed_fnp,sedcor_fnp,bio_np, &
	& d_o_conv_adj,d_dtracer_tot)
      CALL set_d18O(sig_d18O(t_ice),oatracer)
      CALL set_d18O(sig_d18O(t_ice),oatracer_annual)
! update tracer audit changes
      dtracer_tot = dtracer_tot + d_dtracer_tot
! update mean ocean convective adjustments
      o_conv_adj = o_conv_adj + d_o_conv_adj / 1000.0
! asynchronously couple ocean-atmosphere and sediment sub-system (if required)
! update radiocarbon sediment clock (if requested)
! NOTE: force d18O
      IF (runopt(ropt_sed) == yes) THEN
	CALL couple_sed(dt_y,(t_ice - 1),t_loop, &
	  & oatracer%ioat_t,sed_tracer,sed_fnp,frefrac,phys, &
	  & sed,sed_top,sed_par,sed_mix_k,dtracer_tot,sed_fdis,sur_carbchem)
	CALL set_d18O(sig_d18O(t_ice),oatracer)
	IF (sedopt(sopt_sed_14C) == yes) THEN
	  CALL update_sed_14C(sed,sed_top)
	ENDIF
      ENDIF
! asynchronously couple ocean-atmosphere and user-defined sediment cores (if requested)
! update radiocarbon sediment clock (if requested)
      IF (runopt(ropt_sedcor) == yes) THEN
	CALL couple_sedcor(dt_y,(t_ice - 1),t_loop, &
	  & oatracer%ioat_t,sedcor_tracer,sedcor_fnp,frefrac,phys, &
	  & sedcor,sedcor_top,sedcor_par,sed_mix_k,dtracer_tot,sedcor_fdis,sur_carbchem)
	IF (sedopt(sopt_sedcor_14C) == yes) THEN
	  CALL update_sedcor_14C(sedcor,sedcor_top)
	ENDIF
      ENDIF
! check for the occurance of ocean anoxia
! NOTE: if any of the ocean boxes are anoxic:
!       (a) deduct the oxygen deficit from the atmosphere
!       (b) reset [O2] to zero for that particular box
!       (c) reset d18O
      IF (MINVAL(oatracer%ioat_t(:,:,it_O2)) < par_remin_O2_min) THEN
	DO c = 1,n_col
	  DO l = 1,n_lay
	    IF (oatracer%ioat_t(c,l,it_O2) < par_remin_O2_min) THEN
	      IF (runopt(ropt_warn) == yes) THEN
		PRINT*,"WARNING: anoxia in ocean box"
		PRINT*,c,l,oatracer%ioat_t(c,l,it_O2) * 1.0E+06
	      ENDIF
	      program_error              = yes
	      warning(iwarn_anoxia)      = yes
	      oatracer%ioat_a(ia_pO2)    = oatracer%ioat_a(ia_pO2) + &
		& (oatracer%ioat_t(c,l,it_O2) - par_remin_O2_min) * phys(c,l,ip_M) * conv_mol_atm
	      oatracer%ioat_t(c,l,it_O2) = par_remin_O2_min
	    ENDIF
	  END DO
	END DO
	CALL set_d18O(sig_d18O(t_ice),oatracer)
      ENDIF
! update ocean physics temperature array values
      phys(:,:,ip_T) = oatracer%ioat_t(:,:,it_T)

    END DO! (end of sub 1kr loop)

  END DO! (end of main (1kr) loop)

! *************************************************************************************************


! *************************************************************************************************
! *** CLEAN UP ************************************************************************************
! *************************************************************************************************

! *** deallocate array space ***
  IF (sedopt(sopt_sed_diagen_lookup_CaCO3) == yes) THEN
    DEALLOCATE(lookup_sed_dis_cal,STAT=dealloc_error)
    DEALLOCATE(lookup_sed_dis_arg,STAT=dealloc_error)
  ENDIF
  IF (sedopt(sopt_sed_diagen_lookup_opal) == yes) THEN
    DEALLOCATE(lookup_sed_dis_opal,STAT=dealloc_error)
  ENDIF
!!$    DEALLOCATE(dummy_array_REAL,STAT=dealloc_error)

! *** check for problems de-allocating array space ***
  IF (dealloc_error /= 0) THEN
    PRINT*,"FATAL ERROR [sue_main.f90]:"
    PRINT*,"Array space could not be de-allocated"
    STOP
  ENDIF

! *************************************************************************************************


! *************************************************************************************************
! *** SAVE DATA ***********************************************************************************
! *************************************************************************************************

! *** save data to file ***
  PRINT*,""
  PRINT*,"SUE >> saving data ..."
  PRINT*,""
! save kyr np data
  PRINT*,"       * kyr biological new production fluxes"
  CALL save_kyr_bionp_data(t_ice_init, &
    & kyr_phys,kyr_bionp, &
    & spin_phys,spin_bionp)
! save kyr box data (tracer and some ocean box physical characteristics)
  PRINT*,"       * kyr ocean and atmospheric tracers and ocean physics"
  CALL save_kyr_box_data_global(t_ice_init, &
    & kyr_phys,kyr_oatracer,kyr_o_conv_adj, &
    & spin_phys,spin_oatracer,spin_o_conv_adj)
  IF (runopt(ropt_save_box_data_ind) == yes) THEN
    CALL save_kyr_box_data(t_ice_init, &
      & kyr_phys,kyr_oatracer,kyr_o_conv_adj, &
      & spin_phys,spin_oatracer,spin_o_conv_adj)
  END IF
! save kyr sediment sub-system data
  IF (runopt(ropt_sed) == yes) THEN
    PRINT*,"       * kyr sediment sub-system fluxes"
    CALL save_kyr_sed_data_global(t_ice_init,sed_par, &
      & kyr_sed_fnp,kyr_sed_fdis, &
      & spin_sed_fnp,spin_sed_fdis)
    IF (sedopt(sopt_save_sed_data_ind) == yes) THEN
      CALL save_kyr_sed_data(t_ice_init,sed_par, &
	& kyr_sed_fnp,kyr_sed_fdis, &
	& spin_sed_fnp,spin_sed_fdis)
    END IF
  END IF
! save final state diagonistic data
  PRINT*,"       * final model diagnostic data"
  CALL save_diagnostic_data(phys,sed_par,oatracer_annual,bio_np)
! save sediment sub-system diagnostics data (if selected)
  IF (runopt(ropt_sed) == yes) THEN
    PRINT*,"       * sediment sub-system diagnostics data"
    CALL save_sed_diagnostics(sed_top,sed_par,sed_tracer,sed_fnp,sed_fdis)
  END IF
! save sediment sub-system data (if selected)
  IF ((runopt(ropt_sed) == yes) .AND. (sedopt(sopt_save_sedsysdata) == yes)) THEN
    PRINT*,"       * integral model sediment sub-system stacks"
    CALL save_sed_data(sed,sed_top,sed_par)
  ENDIF
! save user defined sediment core data (if selected)
  IF ((runopt(ropt_sed) == yes) .AND. (runopt(ropt_sedcor)) == yes) THEN
    PRINT*,"       * user-definable sediment core stacks"
    CALL save_sedcor_data(sedcor,sedcor_top,sedcor_par)
  ENDIF

! *************************************************************************************************


! *************************************************************************************************
! *** DISPLAY FINAL RESULTS ***********************************************************************
! *************************************************************************************************

! *** print out the results ***
  PRINT*,""
  PRINT*,"SUE >> final model state ..."
  PRINT*,""
  CALL print_finaldata_xxxx(t_ice_init,dtracer_tot_per)
  IF (program_error == yes) THEN
    PRINT*,""
    PRINT*,"SUE >> the following warnings have been issued ..."
    PRINT*,""
    IF (warning(iwarn_Dcalsat) == yes)  &
      & PRINT'("        * warning #01 : depth of calcite saturation horizon   < 0.0")'
    IF (warning(iwarn_Dargsat) == yes)  &
      & PRINT'("        * warning #02 : depth of aragonite saturation horizon < 0.0")'
    IF (warning(iwarn_newsedth) == yes) &
      & PRINT'("        * warning #03 : thickness of accumulating sediment    > ",F8.6)',par_sed_interf_th
    IF (warning(iwarn_traceraudit) == yes) &
      & PRINT'("        * warning #04 : tracer inventories have drifted significantly")'
    IF (warning(iwarn_anoxia) == yes) &
      & PRINT'("        * warning #05 : part of the ocean has gone anoxic")'
    PRINT*,""
  ELSE
    PRINT*,""
    PRINT*,"SUE >> no warnings have been issued"
    PRINT*,""
  ENDIF
  PRINT*,"******************************************************************************"
  PRINT*,""

! *************************************************************************************************


END PROGRAM sue_main
